# Thind JS

Thind-js is a lightweight JavaScript framework designed for building powerful web applications with Webflow. It provides an intuitive API for data management, rendering, creating sortable tables, form handling, and element manipulation.

## Installation

```sh
npm install thind-js
```

## Usage

### Importing Thind

```js
import Thind, { DataStore, Render, SortableTable } from 'thind-js';
```

### Creating a Thind Instance

Declare the attribute you will use globally in the DOM

```js
const thind = new Thind({ attribute: 'data-my-el' });
```

## Modules and Examples

### Element Handling

Thind provides easy access to elements using custom attributes.

```js
// Get a single element
const element = thind.element.get('my-element');
// Get a all elements
const elements = thind.element.getAll('my-elements');
```

### Form Handling

Manage form submissions, validations, and values.

```js
const form = thind.element.get('form');

// Disable the default Webflow submission
thind.form.disable(form); 

//Get value of a specific field by field name
const value = thind.form.value(form, 'username'); 

//Get values of all fields
const values = thind.form.values(form);

//Check if current form is valid, where all required fields are complete
const isValid = thind.form.isValid(form);

//Show Webflow native Form error and pass a custom error message
thind.form.error(form, 'An error occurred');

//Hide the native Webflow form error state
thind.form.errorHide(form);

//Show the Webflow form sucess state
thind.form.success(form, 'showMessage');

//Show the sucess state with custom sucess message
thind.form.success(form, 'showMessage', 'Login Sucessfull');

//Redirect user based on redirect settings of Webflow from
thind.form.success(form, 'redirect');

// Change the text of form button to show the status of form, at end pass true or false to disable or enable the submit buton. True will disable the button.
thind.form.changeSubmitButton(form, 'Submitting...', true);

// Reset the Webflow form. Clear the inputs and hide error & Success state.
thind.form.reset(form);
```

### Page Management

Retrieve page information and URL parameters.

```js
// Get current page name from url
const pageName = thind.page.name();

// Get all page parameters from url
const params = thind.page.parameters();

// Get value of specific page parameter from url
const param = thind.page.parameter('id');
```

### DataStore

Manage your Webflow application state with a reactive data store.

```js
import { DataStore } from 'thind-js';
import { log } from './logs.js';

// Usage example:
const store = new DataStore({ users: [] }, { caching: true });

// Subscribe to changes in users
store.subscribe('users', (newData) => {
  log(JSON.stringify(newData));
});

// Using DataStore's add, update, delete, and get APIs
store.add('users', { name: 'Placeholder' });
store.add('users', { name: 'Dilraj' });
store.add('users', { name: 'Parminder' });
store.add('users', { name: 'Jasbir' });

store.update('users.4', {
  ...store.get('users.4'),
  age: '23',
  test: true,
});

store.update('users.3', {
  ...store.get('users.3'),
  age: '23',
  test: true,
});

store.delete('users.0');
```

Check the demo [here](https://codesandbox.io/p/sandbox/exciting-brook-hk6x5q?file=%2Fsrc%2Findex.ts&layout=%257B%2522sidebarPanel%2522%253A%2522EXPLORER%2522%252C%2522rootPanelGroup%2522%253A%257B%2522direction%2522%253A%2522horizontal%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522id%2522%253A%2522ROOT_LAYOUT%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522clw81wj7m00063b6hutmyh0tx%2522%252C%2522sizes%2522%253A%255Bnull%252Cnull%255D%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522EDITOR%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522id%2522%253A%2522clw81wj7m00023b6hu978vdui%2522%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522SHELLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522id%2522%253A%2522clw81wj7m00033b6hvialeksq%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522DEVTOOLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522id%2522%253A%2522clw81wj7m00053b6h1h5hpavl%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%252C%2522sizes%2522%253A%255B50%252C50%255D%257D%252C%2522tabbedPanels%2522%253A%257B%2522clw81wj7m00023b6hu978vdui%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw81wj7l00013b6h91zazat8%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522FILE%2522%252C%2522filepath%2522%253A%2522%252Fsrc%252Findex.html%2522%252C%2522state%2522%253A%2522IDLE%2522%257D%252C%257B%2522id%2522%253A%2522clw86xnxz00023b6giup5x7l7%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522FILE%2522%252C%2522initialSelections%2522%253A%255B%257B%2522startLineNumber%2522%253A1%252C%2522startColumn%2522%253A1%252C%2522endLineNumber%2522%253A30%252C%2522endColumn%2522%253A1%257D%255D%252C%2522filepath%2522%253A%2522%252Fsrc%252Findex.ts%2522%252C%2522state%2522%253A%2522IDLE%2522%257D%255D%252C%2522id%2522%253A%2522clw81wj7m00023b6hu978vdui%2522%252C%2522activeTabId%2522%253A%2522clw86xnxz00023b6giup5x7l7%2522%257D%252C%2522clw81wj7m00053b6h1h5hpavl%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw81wj7m00043b6hf2yf78sw%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522UNASSIGNED_PORT%2522%252C%2522port%2522%253A0%252C%2522path%2522%253A%2522%252F%2522%257D%255D%252C%2522id%2522%253A%2522clw81wj7m00053b6h1h5hpavl%2522%252C%2522activeTabId%2522%253A%2522clw81wj7m00043b6hf2yf78sw%2522%257D%252C%2522clw81wj7m00033b6hvialeksq%2522%253A%257B%2522tabs%2522%253A%255B%255D%252C%2522id%2522%253A%2522clw81wj7m00033b6hvialeksq%2522%257D%257D%252C%2522showDevtools%2522%253Atrue%252C%2522showShells%2522%253Afalse%252C%2522showSidebar%2522%253Atrue%252C%2522sidebarPanelSize%2522%253A15%257D)

### Data Rendering

Render data to HTML elements.

```js
const dataStore = new DataStore({ users: [] }, { caching: true });

const renderOptions = {
  element: document.getElementById('data-container'),
  data: [{ name: 'Alice' }, { name: 'Bob' }], // Provide data attribute with json data or bind the render instance with a key from Datastore
  dataStore: dataStore,
  key: 'user-data'
  config: {
    props: {
      name: {
        action: (data, element) => {
          element.textContent = data;
        }
      }
    }
  }
};

const render = new Render(renderOptions);


render.update([{ title: '123', complete: false }, { title: 'ABCD', complete: true }]);
render.clear();
render.destroy();
```

Check the demo [here](https://codesandbox.io/p/sandbox/thind-render-js-pg2qvt?layout=%257B%2522sidebarPanel%2522%253A%2522EXPLORER%2522%252C%2522rootPanelGroup%2522%253A%257B%2522direction%2522%253A%2522horizontal%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522id%2522%253A%2522ROOT_LAYOUT%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522clw8gc0oj00063b6glzywgydr%2522%252C%2522sizes%2522%253A%255B100%252C0%255D%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522EDITOR%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522id%2522%253A%2522clw8gc0oj00023b6ghfcq1pp3%2522%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522SHELLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522id%2522%253A%2522clw8gc0oj00033b6gfn7smpd3%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522DEVTOOLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522id%2522%253A%2522clw8gc0oj00053b6gljnh4jaa%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%252C%2522sizes%2522%253A%255B50%252C50%255D%257D%252C%2522tabbedPanels%2522%253A%257B%2522clw8gc0oj00023b6ghfcq1pp3%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw8gc0oi00013b6gzf69bp76%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522FILE%2522%252C%2522filepath%2522%253A%2522%252Fsrc%252Findex.html%2522%257D%255D%252C%2522id%2522%253A%2522clw8gc0oj00023b6ghfcq1pp3%2522%252C%2522activeTabId%2522%253A%2522clw8gc0oi00013b6gzf69bp76%2522%257D%252C%2522clw8gc0oj00053b6gljnh4jaa%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw8gc0oj00043b6gws40jfp8%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522UNASSIGNED_PORT%2522%252C%2522port%2522%253A0%252C%2522path%2522%253A%2522%252F%2522%257D%255D%252C%2522id%2522%253A%2522clw8gc0oj00053b6gljnh4jaa%2522%252C%2522activeTabId%2522%253A%2522clw8gc0oj00043b6gws40jfp8%2522%257D%252C%2522clw8gc0oj00033b6gfn7smpd3%2522%253A%257B%2522tabs%2522%253A%255B%255D%252C%2522id%2522%253A%2522clw8gc0oj00033b6gfn7smpd3%2522%257D%257D%252C%2522showDevtools%2522%253Atrue%252C%2522showShells%2522%253Afalse%252C%2522showSidebar%2522%253Atrue%252C%2522sidebarPanelSize%2522%253A15%257D)

### Sortable Table

Create tables with sortable columns.

```js
const tableConfig = {
  showIndicators: true,
  acsSvg: '<svg>...</svg>',  // Optional SVG code for icon that will be added text to the text of table header on a accessding sort
  descSvg: '<svg>...</svg>',  // Optional SVG code for icon that will be added text to the text of table header on a descending sort
  SvgClass: 'sort-icon' // Optional class to add to the svg icon
};

const sortableTable = new SortableTable('#header-wrapper', '#list-element', 'data-sort', tableConfig);

// Here we are finding headers inside #header-wrapper with attribute data-sort, and then for each found header we find list items that have data-sort="uniqe value"
```

Check the demo [here](https://codesandbox.io/p/sandbox/thind-table-sort-qmx7dv?layout=%257B%2522sidebarPanel%2522%253A%2522EXPLORER%2522%252C%2522rootPanelGroup%2522%253A%257B%2522direction%2522%253A%2522horizontal%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522id%2522%253A%2522ROOT_LAYOUT%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522UNKNOWN%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522clw8g29hj00063b6gqbffu300%2522%252C%2522sizes%2522%253A%255B100%252C0%255D%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522EDITOR%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522EDITOR%2522%252C%2522id%2522%253A%2522clw8g29hj00023b6gwjrdwsdu%2522%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522direction%2522%253A%2522horizontal%2522%252C%2522id%2522%253A%2522SHELLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522SHELLS%2522%252C%2522id%2522%253A%2522clw8g29hj00033b6gfp57eow6%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%257D%252C%257B%2522type%2522%253A%2522PANEL_GROUP%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522direction%2522%253A%2522vertical%2522%252C%2522id%2522%253A%2522DEVTOOLS%2522%252C%2522panels%2522%253A%255B%257B%2522type%2522%253A%2522PANEL%2522%252C%2522contentType%2522%253A%2522DEVTOOLS%2522%252C%2522id%2522%253A%2522clw8g29hj00053b6gdj919cgh%2522%257D%255D%252C%2522sizes%2522%253A%255B100%255D%257D%255D%252C%2522sizes%2522%253A%255B50%252C50%255D%257D%252C%2522tabbedPanels%2522%253A%257B%2522clw8g29hj00023b6gwjrdwsdu%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw8g29hi00013b6gd21y0ynt%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522FILE%2522%252C%2522filepath%2522%253A%2522%252Fsrc%252Findex.html%2522%257D%255D%252C%2522id%2522%253A%2522clw8g29hj00023b6gwjrdwsdu%2522%252C%2522activeTabId%2522%253A%2522clw8g29hi00013b6gd21y0ynt%2522%257D%252C%2522clw8g29hj00053b6gdj919cgh%2522%253A%257B%2522tabs%2522%253A%255B%257B%2522id%2522%253A%2522clw8g29hj00043b6ghnznfkhv%2522%252C%2522mode%2522%253A%2522permanent%2522%252C%2522type%2522%253A%2522UNASSIGNED_PORT%2522%252C%2522port%2522%253A0%252C%2522path%2522%253A%2522%252F%2522%257D%255D%252C%2522id%2522%253A%2522clw8g29hj00053b6gdj919cgh%2522%252C%2522activeTabId%2522%253A%2522clw8g29hj00043b6ghnznfkhv%2522%257D%252C%2522clw8g29hj00033b6gfp57eow6%2522%253A%257B%2522tabs%2522%253A%255B%255D%252C%2522id%2522%253A%2522clw8g29hj00033b6gfp57eow6%2522%257D%257D%252C%2522showDevtools%2522%253Atrue%252C%2522showShells%2522%253Afalse%252C%2522showSidebar%2522%253Atrue%252C%2522sidebarPanelSize%2522%253A15%257D)

